/***********************************************************************
 * This is a library for those cheap 1.8" SPI displays,
 * like the ones that Adafruit sells for 4 times the price than ebay.
 * It is known to work with display with the "red" flag, there are also
 * green and blue ones. Also similar, they might need some tweaking
 * (pixel offsets, different init procedures).
 *
 * An article with more details can be found here:
 * http://enerds.eu/blog/18-spi-lcd-display-10-eur-with-atmega.html
 *
 * This code is based on the work of Limor Fried/Ladyada,
 * many thanks for your work !!
 *
 * I do not like arduinos, especially the arduino-"code",
 * as it does not good besides rapid prototyping,
 * so i prefer developing in real code to stay portable
 * in the "production" phase.
 *
 * Set the Pins and Ports in the headerfile and you are ready to go.
 *
 * I wired the LCD as follows:
 * VCC      +5V
 * GND      GND
 * SCL      PB5 (SCK), Digital Pin 13 on Arduino
 * SDA      PB3 (MOSI), Digital Pin 11 on Arduino
 * RS       , Digital Pin 7 on Arduino
 * RST      PD6, Digital Pin 6 on Arduino
 * CS       GND
 *
 * I wired CS to GND because this is the only SPI device in my current
 * setup.
 *
 * Feel free to contact me: Tobias Weis <mail@enerds.eu>
 */
 
#include "lcd.h"

#define __PROG_TYPES_COMPAT__
#define F_CPU 16000000L
#include "gfx.h"
#include <avr/pgmspace.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <limits.h>


/********************** START SPI STUFF *********************************/
void SPI_begin(void) {
    DDRB |= (1 << SCL);//out
    DDRB |= (1 << SDA);//out
    DDRC |= (1 << RS);//out

    PORTB &= ~(1 << SCL);//lo
    PORTB &= ~(1 << SDA);//lo
    PORTC |= (1 << RS);//hi

    SPCR |= (1 << MSTR) | (1 << SPE);
}

void SPI_end(void) {
    SPCR &= ~(1 << SPE);
}

void SPI_setBitOrder(uint8_t bitOrder) {
    if(bitOrder == LSBFIRST) {
        SPCR |= (1 << DORD);
    } else {
        SPCR &= ~(1 << DORD);
    }
}

void SPI_setDataMode(uint8_t mode) {
    SPCR = (SPCR & ~SPI_MODE_MASK) | mode;
}

void SPI_setClockDivider(uint8_t rate) {
    SPCR = (SPCR & ~SPI_CLOCK_MASK) | (rate & SPI_CLOCK_MASK);
    SPSR = (SPSR & ~SPI_2XCLOCK_MASK) | ((rate >> 2) & SPI_2XCLOCK_MASK);
}
/********************** END SPI STUFF *********************************/

void init(void) {

    SPI_begin();
    SPI_setClockDivider(SPI_CLOCK_DIV4); // 4 MHz (half speed)
    SPI_setBitOrder(MSBFIRST);
    SPI_setDataMode(SPI_MODE0);

}




